<?php

namespace Drupal\nth_mobile_psms\Service;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Http\ClientFactory;
use Drupal\Core\Site\Settings;
use Drupal\nth_mobile_psms\Entity\NthMobilePsmsTransaction;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Class NthMobilePsmsApi.
 *
 * The NthMobilePsmsApi service.
 */
class NthMobilePsmsApi implements NthMobilePsmsApiInterface {

  /**
   * The NTH Mobile Premium SMS service.
   *
   * @var \Drupal\nth_mobile_psms\Service\NthMobilePsmsInterface
   */
  protected $nthMobilePsms;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The HTTP Client Factory.
   *
   * @var \Drupal\Core\Http\ClientFactory
   */
  protected $clientFactory;

  /**
   * The serializer.
   *
   * @var \Symfony\Component\Serializer\SerializerInterface
   */
  private $serializer;

  /**
   * The time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a NthMobilePsmsApi object.
   *
   * @param \Drupal\nth_mobile_psms\Service\NthMobilePsmsInterface $nth_mobile_psms
   *   The NTH Mobile Premium SMS service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Http\ClientFactory $client_factory
   *   The HTTP Client Factory.
   * @param \Symfony\Component\Serializer\SerializerInterface $serializer
   *   The serializer.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time interface.
   */
  public function __construct(NthMobilePsmsInterface $nth_mobile_psms, RequestStack $request_stack, ClientFactory $client_factory, SerializerInterface $serializer, TimeInterface $time) {
    $this->nthMobilePsms = $nth_mobile_psms;
    $this->requestStack = $request_stack;
    $this->clientFactory = $client_factory;
    $this->serializer = $serializer;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public function submitMessage($content, $data, $price, $userId, $language = NULL, $messageRef = NULL, $label = NULL) {
    $uri = $this->nthMobilePsms->getConfig()->get('mt_submission_url');

    // Required params.
    $params = [
      'command' => 'submitMessage',
      'username' => Settings::get('nth_mobile_psms.settings')['username'],
      'password' => Settings::get('nth_mobile_psms.settings')['password'],
      'msisdn' => $data['msisdn'],
      'businessNumber' => $data['businessNumber'],
      'content' => str_replace(' ', '+', $content),
      'price' => $price,
      'sessionId' => $data['sessionId'],
    ];

    // Optional params.
    $params += [
      'keyword' => $data['keyword'],
      'operatorCode' => $data['operatorCode'],
    ];

    // Add language if specified.
    if (isset($language)) {
      $params += [
        'language' => $language,
      ];
    }

    // Add label if specified.
    if (isset($label)) {
      $params += [
        'label' => $label,
      ];
    }

    // Initiate a HTTP client with a timeout of 180 as requested in the
    // documentation.
    $client = $this->clientFactory->fromOptions(['timeout' => 180]);

    try {
      if (isset($data['messageRef'])) {
        $messageRef = $data['messageRef'];
      }

      // Create PSMS transaction for further follow up.
      if (!isset($messageRef)) {
        $values = [
          'title' => $data['content'],
          'status' => 1,
          'msisdn' => $data['msisdn'],
          'business_number' => $data['businessNumber'],
          'content' => $content,
          'price' => $price,
          'operator_code' => $data['operatorCode'],
          'message_state' => 'draft',
          'uid' => $userId,
        ];

        // @todo Use dependency injection.
        $transaction = NthMobilePsmsTransaction::create($values);
        $transaction->save();

        $params['messageRef'] = $transaction->id();
      }
      // Load existing PSMS transaction for further follow up.
      else {
        // @todo Use dependency injection.
        $transaction = NthMobilePsmsTransaction::load($messageRef);
      }

      // Send the message.
      $result = $this->serializer->decode($client->post($uri, [
        'verify' => TRUE,
        'form_params' => $params,
        'headers' => [
          'Content-type' => 'application/x-www-form-urlencoded',
        ],
      ])->getBody()->getContents(), 'xml');

      // Update the PSMS transaction if result returns correct status.
      if ($this->processResultCode($result['res'])) {
        $transaction->set('session_id', $result['res']['sessionId']);
        $transaction->set('message_id', $result['res']['messageId']);
        $transaction->set('message_state', 'pending');
        $transaction->save();
      }
      // Add the messageRef for follow up.
      else {
        $result['retryMessageRef'] = $transaction->id();
      }

      return $result;
    }
    catch (RequestException $e) {
      watchdog_exception('nth_mobile_psms', $e);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function processResultCode(array $result) {
    if ($result['resultCode'] !== '100') {
      $message = new FormattableMarkup(
        'There was an error sending the Premium SMS message. Error details are as follows:<pre>@response</pre>',
        [
          '@response' => print_r($result, TRUE),
        ]
      );

      $this->nthMobilePsms->getLogger()->debug($message);
    }

    return $result['resultCode'] === '100';
  }

}
